% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Recovery Analysis of Damped Spectrally Sparse Signals and Its Relation to MUSIC"
% Information and Inference: A Journal of the IMA, 2020.
% by Shuang Li, Hassan Mansour, and Michael B. Wakin
% This code is used to reproduce Figure 1.


%% Generate data matrix
clc; clear; close all;

f = [0.1 0.4 0.8];         % true frequencies
r = [0.92 0.98 0.85];      % true damping ratios 
K = length(f);             % # of true frequencies, # of active eigenvalues of A(N x N)
N = 50;                    % # of sensors, system dimension
U = randn(N,K);            % eigenvectors of A
U = U*diag(1./norms(U));   % U is the same as Phi in the paper
c = randn(1,K);            % random coefficients
M = 50;                    % # of time samples
Time = 0:M-1;
Time = Time';

v = [r rand(1,M-K)];
vand = fliplr(vander(v));
vand = vand';
r_vand = vand(:,1:K);
Exp = exp(1j*2*pi*Time*f);
Arf = r_vand.*Exp;
chat = c.*norms(Arf);
Arf = Arf*diag(1./norms(Arf));

Data = Arf*diag(chat)*U';   % true data matrix


%% Plot the real part of the first three columns in data matrix
figure
lw = 2;
fs = 15;
subplot(3,1,1)
plot(real(Data(:,1)),'b-','Linewidth',lw);
ylabel('$\mathbf{X}^{\star}(:,1)$','Interpreter','LaTex','FontSize',fs)
title('Real Part','FontSize',fs)
set(gca,'FontSize',fs);
subplot(3,1,2)
plot(real(Data(:,2)),'b-','Linewidth',lw);
ylabel('$\mathbf{X}^{\star}(:,2)$','Interpreter','LaTex','FontSize',fs)
set(gca,'FontSize',fs);
subplot(3,1,3)
plot(real(Data(:,3)),'b-','Linewidth',lw);
xlabel('Samples','FontSize',fs)
ylabel('$\mathbf{X}^{\star}(:,3)$','Interpreter','LaTex','FontSize',fs)
set(gca,'FontSize',fs);

%% Parameters estimation via NN-MUSIC

[Ux, Sx, Vx] = svds(Data);
Khat = rank(Data);
Ux = Ux(:,1:Khat);
Vx = Vx(:,1:Khat);
Q = Ux*Vx';               % dual solution

fgrid = 0:0.01:1.00;
rgrid = 0:0.01:1.00;

Qnorm = zeros(length(fgrid),length(rgrid));
rhogrid = zeros(length(fgrid),length(rgrid));

for i = 1:length(rgrid)
    rg = rgrid(i)*ones(1,length(fgrid));
    if length(fgrid) < M
        v = [rg rand(1,M-length(fgrid))];
        vand = fliplr(vander(v));
        vand = vand';
        r_vand = vand(:,1:length(fgrid));
        Exp = exp(1j*2*pi*Time*fgrid);
        Agrid = r_vand.*Exp;
        Agrid = Agrid*diag(1./norms(Agrid));
        Qnorm(:,i) = norms(Q'*Agrid);
    else
        vand = fliplr(vander(rg));
        vand = vand';
        r_vand = vand(1:M,:);
        Exp = exp(1j*2*pi*Time*fgrid);
        Agrid = r_vand.*Exp;
        Agrid = Agrid*diag(1./norms(Agrid));
        Qnorm(:,i) = norms(Q'*Agrid);
    end
    rhogrid(:,i) = rgrid(i)*exp(1j*2*pi*fgrid);
end


val = 0.999;
step = 1e-4;
[idxf, idxr] = find(Qnorm>=val);
while 1
    if length(idxf) <= K
        break;
    end
    
    val = val+step;
    [idxf, idxr] = find(Qnorm>=val);
    if abs(val-1) <= 1e-7
        step = 0.1*step;
        val = val-step;
    end

    
end

idxrho = find(Qnorm>=val);
Khat = length(idxr);
rhat = rgrid(idxr);      % estimated damping ratios
fhat = fgrid(idxf);      % estimated frequencies


xx = real(exp(2*pi*1i*(0:0.001:1)));
yy = imag(exp(2*pi*1i*(0:0.001:1)));
zz = zeros(1,length(0:0.001:1));


%% Plot the true (r,f) pairs and the estimated (r,f) pairs
figure
lw = 2;
fs = 15;
ms = 5;
plot3(xx,yy,zz,'-k','LineWidth',lw);
grid on;
hold on;
mesh(real(rhogrid),imag(rhogrid),Qnorm);
y3 = plot3(r.*cos(2*pi.*f),r.*sin(2*pi.*f),ones(K,1),'ro','Linewidth',lw,'MarkerSize',ms+4);
y4 = stem3(rhat.*cos(2*pi.*fhat),rhat.*sin(2*pi.*fhat),Qnorm(idxrho),'b*','Linewidth',lw,'MarkerSize',ms-2);
xlabel('$r\cos(2\pi f)$','Interpreter','LaTex','FontSize',fs)
ylabel('$r\sin(2\pi f)$','Interpreter','LaTex','FontSize',fs)
zlabel('$||\mathcal{Q}(r,f)||_2$','Interpreter','LaTex','FontSize',fs);
h = legend([y3,y4], 'Ground Truth', 'NN-MUSIC');
set(gca,'FontSize',fs);
set(h,'FontSize',fs);

