% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Recovery Analysis of Damped Spectrally Sparse Signals and Its Relation to MUSIC"
% Information and Inference: A Journal of the IMA, 2020.
% by Shuang Li, Hassan Mansour, and Michael B. Wakin
% This code is used to reproduce Figure 2.


%% Generate compressed data matrix
clc; clear; close all;

f = [0.8 0.1 0.4];         % true frequencies
r = [0.85 0.92 0.98];      % true damping ratios 
K = length(f);             % # of true frequencies, # of active eigenvalues of A(N x N)
N = 50;                    % # of sensors, system dimension
U = randn(N,K);            % eigenvectors of A
U = U*diag(1./norms(U));
c = [1 2 3];               % coefficients
M = 50;
Time = 0:M-1;
Time = Time';


v = [r rand(1,M-K)];
vand = fliplr(vander(v));
vand = vand';
r_vand = vand(:,1:K);
Exp = exp(1j*2*pi*Time*f);
Arf = r_vand.*Exp;
chat = c.*norms(Arf);
Arf = Arf*diag(1./norms(Arf));

Data = Arf*diag(chat)*U';    % true data matrix

sigma = 0.1;                 % noise level
noise = sigma*randn(M,N); 
NData = Data+noise;          % noisy data matrix
SNR = snr(Data,noise);       % signal noise ratio

%% Plot the real part of the first three columns in noiseless and noisy data matrix
figure
lw = 2;
fs = 15;
ax1 = 0; ax2 = 50; ax3 = -0.5; ax4 = 0.8;
subplot(3,2,2)
plot(real(NData(:,1)),'b-','Linewidth',lw);
ylabel('$\mathbf{Y}(:,1)$','Interpreter','LaTex','FontSize',fs)
title('Real Part','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;
subplot(3,2,4)
plot(real(NData(:,2)),'b-','Linewidth',lw);
ylabel('$\mathbf{Y}(:,2)$','Interpreter','LaTex','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;
subplot(3,2,6)
plot(real(NData(:,3)),'b-','Linewidth',lw);
xlabel('Samples','FontSize',fs)
ylabel('$\mathbf{Y}(:,3)$','Interpreter','LaTex','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;

subplot(3,2,1)
plot(real(Data(:,1)),'b-','Linewidth',lw);
ylabel('$\mathbf{X}^{\star}(:,1)$','Interpreter','LaTex','FontSize',fs)
title('Real Part','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;
subplot(3,2,3)
plot(real(Data(:,2)),'b-','Linewidth',lw);
ylabel('$\mathbf{X}^{\star}(:,2)$','Interpreter','LaTex','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;
subplot(3,2,5)
plot(real(Data(:,3)),'b-','Linewidth',lw);
xlabel('Samples','FontSize',fs)
ylabel('$\mathbf{X}^{\star}(:,3)$','Interpreter','LaTex','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;

%% Nuclear norm denoising

tau=0.2*sigma*sqrt(4*M*N*log(M));

cvx_begin quiet
variable u0;
variable u(M-1) complex;
variable T(M,M) hermitian;
variable Xn(M,N) complex;
variable W(N,N) Hermitian;
dual variable Y0
minimize(0.5*tau*trace(T)+0.5*tau*trace(W)+0.5*sum_square_abs(NData(:)-Xn(:)))
subject to
Y0:[T   Xn; ...
    Xn'   W] == semidefinite(M+N,M+N);
cvx_end
Q_NNM = -2*Y0(1:M,M+1:M+N);
Q_NNM = Q_NNM/tau;

%% Truncate the SVD of $\Q$ to the $K$ largest terms
% Q_NNM = full(Q_NNM);
% [Uq, Sq, Vq] = svd(Q_NNM);
% Uq = Uq(:,1:K);
% Vq = Vq(:,1:K);
% Sq = Sq(1:K,1:K);
% Q_NNM = Uq*Sq*Vq';

% Shuang: no improvement after adding this part.


%% Check whether ||W||<1

[Ux, Sx, Vx] = svd(Data);
Ux = Ux(:,1:K);
Vx = Vx(:,1:K);
Sx = Sx(1:K,1:K);
W = Q_NNM - Ux*Vx';
norm(W)



%% Construct dual polynomial

fgrid = 0:0.01:1;
rgrid = 0:0.01:1;

Qnorm = zeros(length(fgrid),length(rgrid));
rhogrid = zeros(length(fgrid),length(rgrid));

for i = 1:length(rgrid)
    rg = rgrid(i)*ones(1,length(fgrid));
    if length(fgrid) < M
        v = [rg rand(1,M-length(fgrid))];
        vand = fliplr(vander(v));
        vand = vand';
        r_vand = vand(:,1:length(fgrid));
        Exp = exp(1j*2*pi*Time*fgrid);
        Agrid = r_vand.*Exp;
        Agrid = Agrid*diag(1./norms(Agrid));
        Qnorm(:,i) = norms(Q_NNM'*Agrid);
    else
        vand = fliplr(vander(rg));
        vand = vand';
        r_vand = vand(1:M,:);
        Exp = exp(1j*2*pi*Time*fgrid);
        Agrid = r_vand.*Exp;
        Agrid = Agrid*diag(1./norms(Agrid));
        Qnorm(:,i) = norms(Q_NNM'*Agrid);
    end
    rhogrid(:,i) = rgrid(i)*exp(1j*2*pi*fgrid);
end

Qnorm_true = zeros(K,1);
for k = 1:K
    rk = r(k);
    fk = f(k);
    Agrid = (rk.^Time).*exp(1j*2*pi*Time*fk);
    Agrid = Agrid*diag(1./norms(Agrid));
    Qnorm_true(k) = norm(Q_NNM'*Agrid);   
end



%% Show results

sval = 1-0.0003;   % May need to adjust

idx = find(imregionalmax(Qnorm));
idx = idx(2:end);
idxr = fix(idx/length(rgrid))+1;
idxf = rem(idx,length(rgrid));

idxrho = idx;
Khat = length(idxr);
rhat = rgrid(idxr);
fhat = fgrid(idxf);

xx = real(exp(2*pi*1i*(0:0.001:1)));
yy = imag(exp(2*pi*1i*(0:0.001:1)));
zz = zeros(1,length(0:0.001:1));
    

%% Plot the true (r,f) pairs and the estimated (r,f) pairs
figure
lw = 2;
fs = 15;
ms = 5;
plot3(xx,yy,zz,'-k','LineWidth',lw);
grid on;
hold on;
mesh(real(rhogrid),imag(rhogrid),Qnorm);
y3 = stem3(r.*cos(2*pi.*f),r.*sin(2*pi.*f),Qnorm_true,'ro','Linewidth',lw,'MarkerSize',ms+4);
y4 = stem3(rhat.*cos(2*pi.*fhat),rhat.*sin(2*pi.*fhat),Qnorm(idxrho),'b*','Linewidth',lw,'MarkerSize',ms+2);
xlabel('$r\cos(2\pi f)$','Interpreter','LaTex','FontSize',fs)
ylabel('$r\sin(2\pi f)$','Interpreter','LaTex','FontSize',fs)
zlabel('$||\mathcal{Q}(r,f)||_2$','Interpreter','LaTex','FontSize',fs);
h = legend([y3,y4], 'Ground Truth', 'NN-MUSIC');
set(gca,'FontSize',fs);
set(h,'FontSize',fs);















