% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Recovery Analysis of Damped Spectrally Sparse Signals and Its Relation to MUSIC"
% Information and Inference: A Journal of the IMA, 2020.
% by Shuang Li, Hassan Mansour, and Michael B. Wakin
% This code is used to reproduce Figure 5.


%%
clc;clear; close all;

trial=20;                 % # of trials
M=70;                     % # of time samples
FFreq = 0.05:0.05:0.95;   % frequency set
rr = 0.940:0.0025:1;      % damping ratio set
KK = 2:1:19;              % # of frequencies
MMp = 15:3:66;            % # of measurements per sensor
N=50;                     % # of sensors, system dimension
Time=0:M-1;
Time=Time';
fgrid = 0:0.01:1;
rgrid = 0:0.00125:1;
count_para=zeros(size(MMp,2),size(KK,2));
count_data=zeros(size(MMp,2),size(KK,2));
for t=1:trial
    for mm=1:size(MMp,2)
        Mp=MMp(mm);               % number of meansurements per sensor
        for kk=1:size(KK,2)
            K=KK(kk);
            fprintf('Trial=%d,Mp=%d,K=%d\n',t,Mp,K)
            
            ff=randperm(size(FFreq,2),K);
            f=sort(FFreq(ff));    % true frequencies
            r = sort(rr(ff));
 
            U = randn(N,K);        
            U = U*diag(1./norms(U));
            V = pinv(U);
            x0 = randn(N,1);      % initial state
            c = V*x0;
            c = c';               % coefficients
            
            v = [r rand(1,M-K)];
            vand = fliplr(vander(v));
            vand = vand';
            r_vand = vand(:,1:K);
            Exp = exp(1j*2*pi*Time*f);
            Arf = r_vand.*Exp;
            chat = c.*norms(Arf);
            Arf = Arf*diag(1./norms(Arf));
            
            Data = Arf*diag(chat)*U';           % true data matrix        
                        
            indx_me=randperm(N*M,Mp*N);         % observation index
            CData=zeros(M,N);
            CData(indx_me)=Data(indx_me);       % data matrix with random missing entries

            
            % Parameters estimation via MD-MUSIC
            cvx_begin quiet
            variable T(M,M) hermitian;
            variable Xn(M,N) complex;
            variable W(N,N) Hermitian;
            dual variable Y0
            minimize(0.5*trace(T)+0.5*trace(W))
            subject to
            Y0:[T   Xn; ...
                Xn'   W] == semidefinite(M+N,M+N);
            Xn(indx_me) == CData(indx_me);
            cvx_end
            Q_NNM=-2*Y0(1:M,M+1:M+N);

            % Construct dual polynomial           
            Qnorm = zeros(length(fgrid),length(rgrid));
            rhogrid1 = zeros(length(fgrid),length(rgrid));
            
            for i = 1:length(rgrid)
                rg = rgrid(i)*ones(1,length(fgrid));
                if length(fgrid) < M
                    v = [rg rand(1,M-length(fgrid))];
                    vand = fliplr(vander(v));
                    vand = vand';
                    r_vand = vand(:,1:length(fgrid));
                    Exp = exp(1j*2*pi*Time*fgrid);
                    Agrid = r_vand.*Exp;
                    Agrid = Agrid*diag(1./norms(Agrid));
                    Qnorm(:,i) = norms(Q_NNM'*Agrid);
                else
                    vand = fliplr(vander(rg));
                    vand = vand';
                    r_vand = vand(1:M,:);
                    Exp = exp(1j*2*pi*Time*fgrid);
                    Agrid = r_vand.*Exp;
                    Agrid = Agrid*diag(1./norms(Agrid));
                    Qnorm(:,i) = norms(Q_NNM'*Agrid);
                end
                rhogrid1(:,i) = rgrid(i)*exp(1j*2*pi*fgrid);
            end
                        
            val = 0.999;
            step = 1e-4;
            [idxf, idxr] = find(Qnorm>=val);
            while 1
                if length(idxf) <= K
                    break;
                end
                 
                val = val+step;
                [idxf, idxr] = find(Qnorm>=val);
                if abs(val-1) <= 1e-7
                    step = 0.1*step;
                    val = val-step;
                end                
            end
             
            
            [idxf, idxr] = find(Qnorm>=val);
            idxrho = find(Qnorm>=val);
            rhat = sort(rgrid(idxr));
            fhat = sort(fgrid(idxf));

            if length(rhat) == K      
                if max(abs(rhat-r))<=1e-5 && max(abs(fhat-f))<=1e-5
                    count_para(mm,kk)=count_para(mm,kk)+1; % count successful recovery
                end
            end
            if norm(Xn-Data,'fro')/norm(Data,'fro') <= 1e-5
                count_data(mm,kk)=count_data(mm,kk)+1;     % count successful recovery
            end
        end
        count_data
        count_para
    end
end

p_para=count_para./trial;   % probability of successfull recovery for parameters
p_data=count_data./trial;   % probability of successfull recovery for data

%% ==================== Show Results =======================
figure;
fs=22;
imagesc(KK,MMp./M,p_para);axis xy
xlabel('$K$','Interpreter','LaTex','FontSize',fs);
ylabel('$|\Omega|/(MN)$','Interpreter','LaTex','FontSize',fs)
title('Probability of successful recovery for $(r,f)$','Interpreter','LaTex','FontSize',fs-5);
set(gcf, 'Color', 'w');
colormap pink;colorbar;
set(gca,'FontSize',fs);


figure;
imagesc(KK,MMp./M,p_data);axis xy
xlabel('$K$','Interpreter','LaTex','FontSize',fs);
ylabel('$|\Omega|/(MN)$','Interpreter','LaTex','FontSize',fs)
title('Probability of successful recovery for $\mathbf{X}^{\star}$','Interpreter','LaTex','FontSize',fs-5);
set(gcf, 'Color', 'w');
colormap pink;colorbar;
set(gca,'FontSize',fs);


figure;
imagesc(KK,MMp./M,p_para-p_data);axis xy
xlabel('$K$','Interpreter','LaTex','FontSize',fs);
ylabel('$|\Omega|/(MN)$','Interpreter','LaTex','FontSize',fs)
title('Probability difference','FontSize',fs-5);
set(gcf, 'Color', 'w');
colormap pink;colorbar;
set(gca,'FontSize',fs);


