% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Recovery Analysis of Damped Spectrally Sparse Signals and Its Relation to MUSIC"
% Information and Inference: A Journal of the IMA, 2020.
% by Shuang Li, Hassan Mansour, and Michael B. Wakin
% This code is used to reproduce Figure 6 (a,b).


%%
clc; clear; close all;
tic;
trial=10000;                % number of trials
M=50;                       % number of time samples
f1 = 0.1;                   % the first true frequency
r = [1 1];                  % true damping ratios
Delta_f = 0.002:0.002:1/M;  % separation between true frequencies
K = 2;                      % number of true frequencies
Mp = 15;                    % number of measurements per sensor
N=30;                       % number of sensors
Time=0:M-1;
Time=Time';
U = dftmtx(N);       
U = U(:,1:K);
U = U*diag(1./norms(U));
c = [1 1];                  % coefficients
count_data = zeros(1,length(Delta_f));
coherence_data = zeros(trial,length(Delta_f));
error = zeros(trial,length(Delta_f));
normData = zeros(trial,length(Delta_f));
svdData = zeros(3,trial,length(Delta_f));
svdXn = zeros(3,trial,length(Delta_f));


for t=1:trial
    indx_me=randperm(N*M,Mp*N);     % observation index
    for ff=1:length(Delta_f)
        delta_f = Delta_f(ff);
        fprintf('Trial=%d,delta_f=%.4f\n',t,delta_f)
        
        f = [f1 f1+delta_f];        % true frequencies       
        
        Exp = exp(1j*2*pi*Time*f);
        Arf = Exp;
        chat = c.*norms(Arf);
        Arf = Arf*diag(1./norms(Arf));
        
        Data = Arf*diag(chat)*U';   % true data matrix
        [UD, SD, VD] = svd(Data);
        diagSD = diag(SD);
        svdData(:,t,ff) = diagSD(1:3);
        
        
        UD = UD(:,1:K);
        VD = VD(:,1:K);
        mu1 = max(norms(UD').^2)*size(UD,1)/K;
        %%mu2 = max(norms(VD').^2)*size(VD,1)/K;
        coherence_data(t,ff) = mu1;
               
        CData=zeros(M,N);
        CData(indx_me)=Data(indx_me);        % data matrix with random missing entries

        
        % Parameters estimation via MD-MUSIC
        cvx_begin quiet
        variable T(M,M) hermitian;
        variable Xn(M,N) complex;
        variable W(N,N) Hermitian;
        dual variable Y0
        minimize(0.5*trace(T)+0.5*trace(W))
        subject to
        Y0:[T   Xn; ...
            Xn'   W] == semidefinite(M+N,M+N);
        Xn(indx_me) == CData(indx_me);
        cvx_end
        
        error(t,ff) = norm(Xn-Data,'fro');
        normData(t,ff) = norm(Data,'fro');
        
        [UDn, SDn, VDn] = svd(Xn);
        diagSDn = diag(SDn);
        svdXn(:,t,ff) = diagSDn(1:3);
                
        
        if norm(Xn-Data,'fro')/norm(Data,'fro') <= 1e-5
            count_data(ff)=count_data(ff)+1;   % count successful probability
        end
        
        
    end
    count_data
    
end
if trial == 1
    co_data = coherence_data;
else
    co_data = mean(coherence_data);
end
p_data=count_data./trial;   % probability of successfull recovery for data

runtime = toc
%% ==================== Show Results =======================
figure;
fs = 22;
lw = 3;
plot(Delta_f,co_data,'b-o','LineWidth',lw);
xlabel('$\Delta_f$','Interpreter','LaTex','FontSize',fs);
ylabel('$\mu_0^\star$','Interpreter','LaTex','FontSize',fs);
set(gca,'FontSize',fs);
axis tight;


figure;
plot(Delta_f,p_data,'b-o','LineWidth',lw);
xlabel('$\Delta_f$','Interpreter','LaTex','FontSize',fs);
ylabel('Probability of successful recovery','FontSize',fs-1);
set(gca,'FontSize',fs);
axis tight;



