
% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Recovery Analysis of Damped Spectrally Sparse Signals and Its Relation to MUSIC"
% Information and Inference: A Journal of the IMA, 2020.
% by Shuang Li, Hassan Mansour, and Michael B. Wakin
% This code is used to reproduce Figure 6 (e,f).


%%

clc; clear; close all;
tic;
trial=100;               % number of trials
M=50;                    % number of samples
f1 = 0.1;
delta_f = 1/M;           % separation between true frequencies
f = [f1 f1+delta_f];     % frequencies
r1 = 1;                  % damping ratios
R2 = 0.8:0.02:1;  
K = 2;                   % number of frequencies
Mp = 15;                 % number of measurements per sensor
N=30;                    % number of sensors
Time=0:M-1;
Time=Time';
U = dftmtx(N);
U = U(:,1:K);
U = U*diag(1./norms(U));
c = [1 1];

fgrid = 0:0.01:1;
rgrid = 0.6:0.02:1;

count_data = zeros(1,length(R2));
count_r1 = zeros(1,length(R2));
count_r2 = zeros(1,length(R2));
coherence_data = zeros(trial,length(R2));
error = zeros(trial,length(R2));
normData = zeros(trial,length(R2));
svdData = zeros(2,trial,length(R2));
svdXn = zeros(2,trial,length(R2));


for t=1:trial
    indx_me=randperm(N*M,Mp*N);
    for ri=1:length(R2)
        r2 = R2(ri);
        
        fprintf('Trial=%d,r2=%.2f\n',t,r2)
        
        r = [r1 r2];       % damping ratios
        v = [r rand(1,M-K)];
        vand = fliplr(vander(v));
        vand = vand';
        r_vand = vand(:,1:K);
        
        Exp = exp(1j*2*pi*Time*f);
        Arf = r_vand.*Exp;
        chat = c.*norms(Arf);
        Arf = Arf*diag(1./norms(Arf));
        
        Data = Arf*diag(chat)*U';
        [UD, SD, VD] = svd(Data);
        diagSD = diag(SD);
        svdData(:,t,ri) = diagSD(1:2);
        
        
        UD = UD(:,1:K);
        VD = VD(:,1:K);
        mu1 = max(norms(UD').^2)*size(UD,1)/K;
        coherence_data(t,ri) = mu1;
        
        CData=zeros(M,N);
        CData(indx_me)=Data(indx_me);        % Missing entries
        
        % Nuclear norm minimization
        cvx_begin quiet
        variable T(M,M) hermitian;
        variable Xn(M,N) complex;
        variable W(N,N) Hermitian;
        dual variable Y0
        minimize(0.5*trace(T)+0.5*trace(W))
        subject to
        Y0:[T   Xn; ...
            Xn'   W] == semidefinite(M+N,M+N);
        Xn(indx_me) == CData(indx_me);
        cvx_end
        
        error(t,ri) = norm(Xn-Data,'fro');
        normData(t,ri) = norm(Data,'fro');
        
        [UDn, SDn, VDn] = svd(Xn);
        diagSDn = diag(SDn);
        svdXn(:,t,ri) = diagSDn(1:2);
        
        
        if norm(Xn-Data,'fro')/norm(Data,'fro') <= 1e-5
            count_data(ri)=count_data(ri)+1;
        end
        
        
        % ===================
        
        Q_NNM=-2*Y0(1:M,M+1:M+N);
        
        % Construct dual polynomial
        Qnorm = zeros(length(fgrid),length(rgrid));
        rhogrid1 = zeros(length(fgrid),length(rgrid));
        
        for i = 1:length(rgrid)
            rg = rgrid(i)*ones(1,length(fgrid));
            if length(fgrid) < M
                v = [rg rand(1,M-length(fgrid))];
                vand = fliplr(vander(v));
                vand = vand';
                r_vand = vand(:,1:length(fgrid));
                Exp = exp(1j*2*pi*Time*fgrid);
                Agrid = r_vand.*Exp;
                Agrid = Agrid*diag(1./norms(Agrid));
                Qnorm(:,i) = norms(Q_NNM'*Agrid);
            else
                vand = fliplr(vander(rg));
                vand = vand';
                r_vand = vand(1:M,:);
                Exp = exp(1j*2*pi*Time*fgrid);
                Agrid = r_vand.*Exp;
                Agrid = Agrid*diag(1./norms(Agrid));
                Qnorm(:,i) = norms(Q_NNM'*Agrid);
            end
            rhogrid1(:,i) = rgrid(i)*exp(1j*2*pi*fgrid);
        end
        
        val = 0.999;
        step = 1e-4;
        [idxf, idxr] = find(Qnorm>=val);
        while 1
            if length(idxf) <= K
                break;
            end
            
            val = val+step;
            [idxf, idxr] = find(Qnorm>=val);
            if abs(val-1) <= 1e-7
                step = 0.1*step;
                val = val-step;
            end
        end
        %val
        
        [idxf, idxr] = find(Qnorm>=val);
        idxrho = find(Qnorm>=val);
        rhat = sort(rgrid(idxr),'descend');
        
        if ~isempty(find(abs(rhat-r1) <= 1e-5))
            count_r1(ri)=count_r1(ri)+1;
        end
        
        if ~isempty(find(abs(rhat-r2) <= 1e-5))
            count_r2(ri)=count_r2(ri)+1;
        end
        
        
    end
    count_data
    count_r1
    count_r2
end
if trial == 1
    co_data = coherence_data;
else
    co_data = mean(coherence_data);
end
p_data=count_data./trial;   % probability of successfull recovery for data
p_r1=count_r1./trial;       % probability of successfull recovery for r1
p_r2=count_r2./trial;       % probability of successfull recovery for r2

runtime = toc
%% ==================== Show Results =======================
figure(1); clf;
fs = 22;
lw = 3;
plot(R2,co_data,'b-o','LineWidth',lw);
xlabel('$r_2$','Interpreter','LaTex','FontSize',fs);
ylabel('$\mu_0^\star$','Interpreter','LaTex','FontSize',fs);
set(gcf, 'Color', 'w');
set(gca,'FontSize',fs);
axis tight;


figure(2); clf;
hold on;
plot(R2,p_data,'b-o','LineWidth',lw);
plot(R2,p_r1,'b--','LineWidth',lw);
plot(R2,p_r2,'b:','LineWidth',lw);
xlabel('$r_2$','Interpreter','LaTex','FontSize',fs);
ylabel('Probability of successful recovery','FontSize',fs-1);
h = legend('$\mathbf{X}^\star$', '$r_1$', '$r_2$','Interpreter','LaTex','FontSize',fs);
set(gcf, 'Color', 'w');
set(gca,'FontSize',fs);
axis tight;
box on;
grid on;



