% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Recovery Analysis of Damped Spectrally Sparse Signals and Its Relation to MUSIC"
% Information and Inference: A Journal of the IMA, 2020.
% by Shuang Li, Hassan Mansour, and Michael B. Wakin
% This code is used to reproduce Figure 7(a).



%%
clc; clear; close all;

trial = 100;                   % number of trials
f = [0.1 0.16 0.8];            % true frequencies
r = [0.86 0.92 0.98];          % true damping ratios 
K = length(r);                 % # of true frequencies, # of active eigenvalues of A(N x N)
N = 10;                        % # of sensors, system dimension

MM = 5:5:50;                   % # of uniform time samples

fgrid = 0:0.02:1.00;
rgrid = 0:0.02:1.00;

count_MUSIC = zeros(1,length(MM));
count_ANM = zeros(1,length(MM));


for t=1:trial
    for mm=1:length(MM)
        M = MM(mm);
        fprintf('Trial=%d,M=%d\n',t,M)
        
        Time = 0:M-1;
        Time = Time';
        U = randn(N,K);               % eigenvectors of A
        U = U*diag(1./norms(U));
        c = randn(1,K);               % random coefficients
        v = [r rand(1,M-K)];
        vand = fliplr(vander(v));
        vand = vand';
        r_vand = vand(:,1:K);
        Exp = exp(1j*2*pi*Time*f);
        Arf = r_vand.*Exp;
        chat = c.*norms(Arf);
        Arf = Arf*diag(1./norms(Arf));
        
        Data = Arf*diag(chat)*U';   % true data matrix
   
        
        % =========== Parameters estimation via NN-MUSIC ===========
        
        [Ux, Sx, Vx] = svds(Data);
        Khat = rank(Data);
        Ux = Ux(:,1:Khat);
        Vx = Vx(:,1:Khat);
        Q = Ux*Vx';               % dual solution
               
        Qnorm = zeros(length(fgrid),length(rgrid));
        rhogrid = zeros(length(fgrid),length(rgrid));
        
        for i = 1:length(rgrid)
            rg = rgrid(i)*ones(1,length(fgrid));
            if length(fgrid) < M
                v = [rg rand(1,M-length(fgrid))];
                vand = fliplr(vander(v));
                vand = vand';
                r_vand = vand(:,1:length(fgrid));
                Exp = exp(1j*2*pi*Time*fgrid);
                Agrid = r_vand.*Exp;
                Agrid = Agrid*diag(1./norms(Agrid));
                Qnorm(:,i) = norms(Q'*Agrid);
            else
                vand = fliplr(vander(rg));
                vand = vand';
                r_vand = vand(1:M,:);
                Exp = exp(1j*2*pi*Time*fgrid);
                Agrid = r_vand.*Exp;
                Agrid = Agrid*diag(1./norms(Agrid));
                Qnorm(:,i) = norms(Q'*Agrid);
            end
            rhogrid(:,i) = rgrid(i)*exp(1j*2*pi*fgrid);
        end
        
        
        val = 0.98;
        step = 1e-4;
        [idxf, idxr] = find(Qnorm>=val);
        while 1
            if length(idxf) <= K
                break;
            end            
            val = val+step;
            [idxf, idxr] = find(Qnorm>=val);
            if abs(val-1) <= 1e-7
                step = 0.1*step;
                val = val-step;
            end          
        end
        
        idxrho = find(Qnorm>=val);
        Khat = length(idxr);
        f_NN_MUSIC = fgrid(idxf);     % estimated frequencies
        r_NN_MUSIC = rgrid(idxr);     % estimated damping ratios
        f_NN_MUSIC = sort(f_NN_MUSIC);
        
        % =========== Parameters estimation via ANM ===========
        
        
        cvx_begin quiet
        variable u0;
        variable u(M-1) complex;
        variable Tu(M,M) hermitian;
        variable X(M,N) complex;
        variable W(N,N) Hermitian;
        dual variable Y0
        minimize(0.5/M*trace(Tu)+0.5*trace(W))
        subject to
        Tu == toeplitz([u0;conj(u)],[u0;conj(u)]');
        Y0:[Tu   X; ...
            X'   W] == semidefinite(M+N,M+N);
        X==Data;
        cvx_end
        
        Y=-2*Y0(1:M,M+1:M+N);          % dual solution
        a=exp(1j*2*pi*Time*fgrid);
        Q=Y'*a;                        % dual polynomial
        if N > 1
            NormQ=norms(Q);
        else
            NormQ=abs(Q);
        end
        
        aval = 1e-3;
        astep = 0.1;
        [pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',1-aval);
        while 1
            if length(locs) <= K
                break;
            end
            aval = aval*astep;
            [pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',1-aval);
            if abs(aval) <= 1e-15
                break;
            end
        end
        
          
        f_ANM=fgrid(locs);      % estimated frequencies
        f_ANM = sort(f_ANM);
                
        if length(f_NN_MUSIC) == K && max(abs(f-f_NN_MUSIC)) <= 1e-5
            count_MUSIC(mm)=count_MUSIC(mm)+1;   % count successful probability
        end
        if length(f_ANM) == K && max(abs((f-f_ANM))) <= 1e-5
            count_ANM(mm)=count_ANM(mm)+1;   % count successful probability
        end
        
        
        
    end
    count_MUSIC
    count_ANM
    
end

p_MUSIC=count_MUSIC./trial;   % probability of successfull recovery with NN-MUSIC
p_ANM=count_ANM./trial;       % probability of successfull recovery with ANM

%% ==================== Show Results =======================


figure;
lw = 5;
fs = 25;
plot(MM,p_MUSIC,'r-','LineWidth',lw); hold on;
plot(MM,p_ANM,'b--','LineWidth',lw); hold on;
xlabel('$M$','Interpreter','LaTex','FontSize',fs);
ylabel('Probability of successful recovery','FontSize',fs-1);
xlim([min(MM) max(MM)])
ylim([0 1]);
h = legend('NN-MUSIC','ANM');
set(gca,'FontSize',fs);
set(h,'FontSize',fs);





