% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Recovery Analysis of Damped Spectrally Sparse Signals and Its Relation to MUSIC"
% Information and Inference: A Journal of the IMA, 2020.
% by Shuang Li, Hassan Mansour, and Michael B. Wakin
% This code is used to reproduce Figure 8(b,d).



%%
clc; clear; close all;

trial = 100;                  % number of trials
M = 20;                       % number of time samples
f1 = 0.1;                     % the first true frequency
r = [0.92 0.98];              % true damping ratios
Delta_f = 0.01:0.01:0.1;      % separation between true frequencies
K = length(r);                % # of true frequencies, # of active eigenvalues of A(N x N)
N = 10;                       % # of sensors, system dimension

Time = 0:M-1;
Time = Time';

fgrid = 0:0.01:1.00;
rgrid = 0:0.02:1.00;
sr = 0.8;                     % sampling rate (20% are missing)
Mp = sr*M;
count_MUSIC = zeros(1,length(Delta_f));
count_ANM = zeros(1,length(Delta_f));
for t=1:trial
    for ff=1:length(Delta_f)
        delta_f = Delta_f(ff);
        fprintf('Trial=%d,delta_f=%.4f\n',t,delta_f)
        
        f = [f1 f1+delta_f];          % true frequencies
        U = randn(N,K);               % eigenvectors of A
        U = U*diag(1./norms(U));
        c = randn(1,K);               % random coefficients
        v = [r rand(1,M-K)];
        vand = fliplr(vander(v));
        vand = vand';
        r_vand = vand(:,1:K);
        Exp = exp(1j*2*pi*Time*f);
        Arf = r_vand.*Exp;
        chat = c.*norms(Arf);
        Arf = Arf*diag(1./norms(Arf));
        
        Data = Arf*diag(chat)*U';       % true data matrix
          
        indx_me=randperm(N*M,Mp*N);     % observation index
        CData=zeros(M,N);
        CData(indx_me)=Data(indx_me);   % data matrix with random missing entries
        
        % =========== Parameters estimation via MD-MUSIC ===========
        cvx_begin quiet
        variable T(M,M) hermitian;
        variable Xn(M,N) complex;
        variable W(N,N) Hermitian;
        dual variable Y0
        minimize(0.5*trace(T)+0.5*trace(W))
        subject to
        Y0:[T   Xn; ...
            Xn'   W] == semidefinite(M+N,M+N);
        Xn(indx_me) == CData(indx_me);
        cvx_end
        Q_NNM=-2*Y0(1:M,M+1:M+N);
        
        ErrX_NNM = norm(Data-Xn,'fro')/norm(Data,'fro');
        
        % Construct dual polynomial
               
        Qnorm = zeros(length(fgrid),length(rgrid));
        rhogrid = zeros(length(fgrid),length(rgrid));
        
        for i = 1:length(rgrid)
            rg = rgrid(i)*ones(1,length(fgrid));
            if length(fgrid) < M
                v = [rg rand(1,M-length(fgrid))];
                vand = fliplr(vander(v));
                vand = vand';
                r_vand = vand(:,1:length(fgrid));
                Exp = exp(1j*2*pi*Time*fgrid);
                Agrid = r_vand.*Exp;
                Agrid = Agrid*diag(1./norms(Agrid));
                Qnorm(:,i) = norms(Q_NNM'*Agrid);
            else
                vand = fliplr(vander(rg));
                vand = vand';
                r_vand = vand(1:M,:);
                Exp = exp(1j*2*pi*Time*fgrid);
                Agrid = r_vand.*Exp;
                Agrid = Agrid*diag(1./norms(Agrid));
                Qnorm(:,i) = norms(Q_NNM'*Agrid);
            end
            rhogrid(:,i) = rgrid(i)*exp(1j*2*pi*fgrid);
        end
        
        
        
        val = 0.98;
        step = 1e-4;
        [idxf, idxr] = find(Qnorm>=val);
        while 1
            if length(idxf) <= K
                break;
            end
            val = val+step;
            [idxf, idxr] = find(Qnorm>=val);
            if abs(val-1) <= 1e-7
                step = 0.1*step;
                val = val-step;
            end
        end
        
        idxrho = find(Qnorm>=val);
        Khat = length(idxr);
        f_NN_MUSIC = fgrid(idxf);     % estimated frequencies
        r_NN_MUSIC = rgrid(idxr);     % estimated damping ratios
        f_NN_MUSIC = sort(f_NN_MUSIC);
        r_NN_MUSIC = sort(r_NN_MUSIC);
        
        % =========== Parameters estimation via ANM ===========
        
        
        cvx_begin quiet
        variable u0;
        variable u(M-1) complex;
        variable Tu(M,M) hermitian;
        variable X(M,N) complex;
        variable W(N,N) Hermitian;
        dual variable Y0
        minimize(0.5/M*trace(Tu)+0.5*trace(W))
        subject to
        Tu == toeplitz([u0;conj(u)],[u0;conj(u)]');
        Y0:[Tu   X; ...
            X'   W] == semidefinite(M+N,M+N);
        X(indx_me) == CData(indx_me);
        cvx_end
        
        Y=-2*Y0(1:M,M+1:M+N);       % dual solution
        a=exp(1j*2*pi*Time*fgrid);
        Q=Y'*a;                     % dual polynomial
        if N > 1
            NormQ=norms(Q);
        else
            NormQ=abs(Q);
        end
        
        aval = 1e-3;
        astep = 0.1;
        [pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',1-aval);
        while 1
            if length(locs) <= K
                break;
            end
            aval = aval*astep;
            [pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',1-aval);
            if abs(aval) <= 1e-15
                break;
            end
        end
        f_ANM=fgrid(locs);      % estimated frequencies
        f_ANM = sort(f_ANM);
                
        if length(f_NN_MUSIC) == K && max(abs((f-f_NN_MUSIC))) <= 1e-5
            count_MUSIC(ff)=count_MUSIC(ff)+1;   % count successful probability
        end
        if length(f_ANM) == K && max(abs((f-f_ANM))) <= 1e-5
            count_ANM(ff)=count_ANM(ff)+1;   % count successful probability
        end
        
        
    end
    count_MUSIC
    count_ANM
    
end

p_MUSIC=count_MUSIC./trial;   % probability of successfull recovery with NN-MUSIC
p_ANM=count_ANM./trial;       % probability of successfull recovery with ANM

%% ==================== Show Results =======================


figure;
lw = 5;
fs = 25;
plot(Delta_f,p_MUSIC,'r-','LineWidth',lw); hold on;
plot(Delta_f,p_ANM,'b--','LineWidth',lw); hold on;
xlabel('$\Delta_f$','Interpreter','LaTex','FontSize',fs);
ylabel('Probability of successful recovery','FontSize',fs-1);
xlim([min(Delta_f) max(Delta_f)])
ylim([0 1]);
h = legend('MD-MUSIC','ANM');
set(gca,'FontSize',fs);
set(h,'FontSize',fs);



