% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Recovery Analysis of Damped Spectrally Sparse Signals and Its Relation to MUSIC"
% Information and Inference: A Journal of the IMA, 2020.
% by Shuang Li, Hassan Mansour, and Michael B. Wakin
% This code is used to reproduce Table 1.


%% Generate data matrix
clc; clear; close all;

f = [0.1 0.4 0.8];          % true frequencies
r = [0.92 0.98 0.85];       % true damping ratios
K = length(f);              % # of true frequencies, # of active eigenvalues of A(N x N)
N = 20;                     % # of sensors, system dimension
M = 50;                     % # of time samples
Time = 0:M-1;
Time = Time';
sr = 0.9;                   % sampling rate: 0.9, 0.8, 0.7, 0.6
Mp = M*sr;                  % number of meansurement per sensor
Trial = 1000;

fgrid = 0:0.01:1;
rgrid = 0:0.01:1;

count_MD_MUSIC = 0;
count_NNM_MUSIC = 0;
count_NNM_ESPRIT = 0;

for t = 1:Trial
    fprintf('Trial=%d\n',t)
    U = randn(N,K);
    U = U*diag(1./norms(U));
    V = pinv(U);
    x0 = randn(N,1);            % initial state
    c = V*x0;
    c = c';                     % coefficients
    
    v = [r rand(1,M-K)];
    vand = fliplr(vander(v));
    vand = vand';
    r_vand = vand(:,1:K);
    Exp = exp(1j*2*pi*Time*f);
    Arf = r_vand.*Exp;
    chat = c.*norms(Arf);
    Arf = Arf*diag(1./norms(Arf));
    
    Data = Arf*diag(chat)*U';       % true data matrix
    
    indx_me=randperm(N*M,Mp*N);     % observation index
    CData=zeros(M,N);
    CData(indx_me)=Data(indx_me);   % data matrix with random missing entries
    
    
    % =========== Parameters estimation via MD-MUSIC ===========
    cvx_begin quiet
    variable T(M,M) hermitian;
    variable Xn(M,N) complex;
    variable W(N,N) Hermitian;
    dual variable Y0
    minimize(0.5*trace(T)+0.5*trace(W))
    subject to
    Y0:[T   Xn; ...
        Xn'   W] == semidefinite(M+N,M+N);
    Xn(indx_me) == CData(indx_me);
    cvx_end
    Q_NNM=-2*Y0(1:M,M+1:M+N);
    
    ErrX_NNM = norm(Data-Xn,'fro')/norm(Data,'fro');
    
    % Construct dual polynomial
    Qnorm1 = zeros(length(fgrid),length(rgrid));
    rhogrid1 = zeros(length(fgrid),length(rgrid));
    
    for i = 1:length(rgrid)
        rg = rgrid(i)*ones(1,length(fgrid));
        if length(fgrid) < M
            v = [rg rand(1,M-length(fgrid))];
            vand = fliplr(vander(v));
            vand = vand';
            r_vand = vand(:,1:length(fgrid));
            Exp = exp(1j*2*pi*Time*fgrid);
            Agrid = r_vand.*Exp;
            Agrid = Agrid*diag(1./norms(Agrid));
            Qnorm1(:,i) = norms(Q_NNM'*Agrid);
        else
            vand = fliplr(vander(rg));
            vand = vand';
            r_vand = vand(1:M,:);
            Exp = exp(1j*2*pi*Time*fgrid);
            Agrid = r_vand.*Exp;
            Agrid = Agrid*diag(1./norms(Agrid));
            Qnorm1(:,i) = norms(Q_NNM'*Agrid);
        end
        rhogrid1(:,i) = rgrid(i)*exp(1j*2*pi*fgrid);
    end
    
    
    sval = 0.99;
    step = 1e-4;
    [idxf, idxr] = find(Qnorm1>=sval);
    
    for ii = 1:1e10
        if length(idxf) <= K
            break;
        end
        sval = sval+step;
        [idxf, idxr] = find(Qnorm1>=sval);
        if abs(sval-1) <= 1e-7
            step = 0.1*step;
            sval = sval-step;
        end
    end
    
    
    f_MD_MUSIC = fgrid(idxf);     % estimated frequencies
    r_MD_MUSIC = rgrid(idxr);     % estimated damping ratios
    
    
    
    if length(r_MD_MUSIC) == K
        if max(abs(sort(r_MD_MUSIC)-sort(r)))<=1e-5 && max(abs(sort(f_MD_MUSIC)-sort(f)))<=1e-5
            count_MD_MUSIC = count_MD_MUSIC+1; % count successful recovery
        end
    end
    
    
    % =========== Parameters estimation via NNM+MUSIC ===========
    
    [Ux, Sx, Vx] = svd(Xn);
    %Khat = length(find(diag(Sx)>=1e-2*max(diag(Sx))));
    Khat = K;
    Ux = Ux(:,1:Khat);
    Vx = Vx(:,1:Khat);
    Q = Ux*Vx';
    
    if Khat == K
        
        Qnorm = zeros(length(fgrid),length(rgrid));
        rhogrid = zeros(length(fgrid),length(rgrid));
        
        for i = 1:length(rgrid)
            rg = rgrid(i)*ones(1,length(fgrid));
            if length(fgrid) < M
                v = [rg rand(1,M-length(fgrid))];
                vand = fliplr(vander(v));
                vand = vand';
                r_vand = vand(:,1:length(fgrid));
                Exp = exp(1j*2*pi*Time*fgrid);
                Agrid = r_vand.*Exp;
                Agrid = Agrid*diag(1./norms(Agrid));
                Qnorm(:,i) = norms(Q'*Agrid);
            else
                vand = fliplr(vander(rg));
                vand = vand';
                r_vand = vand(1:M,:);
                Exp = exp(1j*2*pi*Time*fgrid);
                Agrid = r_vand.*Exp;
                Agrid = Agrid*diag(1./norms(Agrid));
                Qnorm(:,i) = norms(Q'*Agrid);
            end
            rhogrid(:,i) = rgrid(i)*exp(1j*2*pi*fgrid);
        end
        
        
        val = 0.99;
        step = 1e-4;
        [idxf, idxr] = find(Qnorm>=val);
        for ii = 1:1e10
            if length(idxf) <= Khat
                break;
            end
            val = val+step;
            [idxf, idxr] = find(Qnorm>=val);
            if abs(val-1) <= 1e-7
                step = 0.1*step;
                val = val-step;
            end
        end
        
        
        r_NNM_MUSIC = rgrid(idxr);      % estimated damping ratios
        f_NNM_MUSIC = fgrid(idxf);      % estimated frequencies
        
        
        if length(r_NNM_MUSIC) == K
            if max(abs(sort(r_NNM_MUSIC)-sort(r)))<=1e-5 && max(abs(sort(f_NNM_MUSIC)-sort(f)))<=1e-5
                count_NNM_MUSIC = count_NNM_MUSIC+1; % count successful recovery
            end
        end
        
        % =========== Parameters estimation via NNM+ESPRIT ===========
        
        U0 = Ux(1:end-1,:);
        U1 = Ux(2:end,:);
        Psi = pinv(U0)*U1;
        EV_Psi = eig(Psi);
        
        r_NNM_ESPRIT = abs(EV_Psi)';             % estimated damping ratios
        f_NNM_ESPRIT = angle(EV_Psi)/2/pi;      % estimated frequencies
        for fi = 1:Khat
            if f_NNM_ESPRIT(fi) < 0
                f_NNM_ESPRIT(fi) = f_NNM_ESPRIT(fi)+1;
            end
        end
        f_NNM_ESPRIT = f_NNM_ESPRIT';
        
        
        if length(r_NNM_ESPRIT) == K
            if max(abs(sort(r_NNM_ESPRIT)-sort(r)))<=1e-5 && max(abs(sort(f_NNM_ESPRIT)-sort(f)))<=1e-5
                count_NNM_ESPRIT = count_NNM_ESPRIT+1; % count successful recovery
            end
        end
        
    end
    count_MD_MUSIC
    count_NNM_MUSIC
    count_NNM_ESPRIT
end
















